import { reactive, ref } from 'vue'

/** 钩子支持的名称 */
const hooksName: HooksNameType = {
    beforeSubmitHandler: 'beforeSubmitHandler',
    beforedisabledHandler: 'beforedisabledHandler',
    afterdisabledHandler: 'afterdisabledHandler',
    beforeAddHandler: 'beforeAddHandler',
    beforeEditHandler: 'beforeEditHandler',
    beforeQueryHandler: 'beforeQueryHandler',
    afterQueryHandler: 'afterQueryHandler',
    beforeDeteleHandler: 'beforeDeteleHandler',
    afterDeteleHandler: 'afterDeteleHandler',
    beforeBatchDeteleHandler: 'beforeBatchDeteleHandler',
    afterBatchDeteleHandler: 'afterBatchDeteleHandler',
    beforeTableLazyHandler: 'beforeTableLazyHandler'
}

/**
 * 注册钩子
 * @param hooks 组件注册的钩子对象
 * @returns 返回钩子处理函数及钩子名称
 */
function useHooks(hooks: HooksConfigType = { debug: false }) {
    // 当前实例的钩子
    const state: any = reactive({
        hooks: {}
    })

    // 向当前实例注册钩子
    Object.keys(hooks).forEach((item) => {
        if (item !== 'debug') {
            state.hooks[item] = hooks[item]
        }
    })

    /**
     * 处理钩子函数
     * @param hookName 钩子名称
     * @param data 钩子接收的响应式数据
     * @returns 返回处理结果
     */
    const hookHandler = (hookName: string, data: { CrudState: any; Data: any }) => {
        let res = false
        try {
            if (hooks.debug) {
                console.group('DeBug：', hookName)
                console.table(data.Data)
                console.groupEnd()
            }
            // 自定义的返回值是什么
            res = state.hooks[hookName](data)
        } catch (error) {
            res = true
        }
        return res
    }

    return {
        hookHandler
    }
}

export { hooksName, useHooks }
