import { ElNotification } from 'element-plus'

/***********************************   方法   *********************************** */

/** 初始化通知 */
const CRUD_Notification = (title: string, type: 'success' | 'warning' | 'info' | 'error' = 'success') => {
    ElNotification({
        title,
        type,
        offset: 70,
        duration: 2500
    })
}
/**
 * 深拷贝
 * @param Obj 需要拷贝的数据对象
 * @returns 一个新的数据对象
 */
function DeepClone(Obj: any) {
    let newObj: any

    try {
        newObj = Array.isArray(Obj) ? [] : {}
        if (Obj === null) newObj = null
    } catch (error) {
        newObj = {}
    }

    for (const attr in Obj) {
        if (typeof Obj[attr] === 'object') {
            newObj[attr] = DeepClone(Obj[attr])
        } else {
            newObj[attr] = Obj[attr]
        }
    }

    return newObj
}

/** 数据合并 */
function DataMerger(...args: any) {
    /** 判断对象是否是一个纯粹的对象 */
    function isPlainObject(obj: any) {
        return typeof obj === 'object' && Object.prototype.toString.call(obj) === '[object Object]'
    }

    let len = arguments.length,
        target = arguments[0]
    if (!isPlainObject(target)) {
        target = {}
    }

    for (let i = 1; i < len; i++) {
        let source = arguments[i]

        if (isPlainObject(source)) {
            for (let key in source) {
                if (key === '__proto__' || target === source[key]) {
                    continue
                }

                // ADD：判断原值中是否包含要合并的key
                if (target.hasOwnProperty(key)) {
                    if (isPlainObject(source[key])) {
                        target[key] = DataMerger(target[key], source[key])
                    } else {
                        target[key] = source[key]
                    }
                }
            }
        }
    }
    return target
    // console.log(Obj1, Obj2)

    // let newObj = DeepClone(Obj1)

    // for (var key in Obj2) {
    //     if (newObj.hasOwnProperty(key)) {
    //         if (typeof Obj2[key] === 'object') {
    //             newObj[key] = DataMerger(newObj[key], Obj2[key])
    //         } else {
    //             newObj[key] = Obj2[key]
    //         }
    //     } else {
    //         newObj[key] = Obj2[key]
    //     }
    // }
    // console.log(newObj)

    // return newObj
}

/**
 * 比较两个 Obj 的值是否修改
 * @param obj1 原obj
 * @param obj2 新obj
 * @returns 修改过的obj
 */
function isObjEqual(obj1: any, obj2: any) {
    let res: any = {}

    let aProps = Object.getOwnPropertyNames(obj1)
    let bProps = Object.getOwnPropertyNames(obj2)

    if (aProps.length === bProps.length) {
        //循环拿到数值进行对比
        aProps.forEach((e) => {
            if (obj1[e] !== obj2[e]) res[e] = obj2[e]
        })
    }

    return res
}

/***********************************   方法   *********************************** */

export { DeepClone, DataMerger, isObjEqual, CRUD_Notification }
