/// <reference types="vite/client" />

declare module '*.vue' {
    import type { DefineComponent } from 'vue'
    const component: DefineComponent<{}, {}, any>
    export default component
}
/** 组件参数类型 */
interface CrusStateType {
    /** 汇总组件Ref状态 */
    CompRef: any
    /** 当前操作是新增还是编辑 */
    Mode: Ref<string>
    /** 新增/编辑对话框可见 */
    DialogVisible: Ref<boolean>
    /** 表格加载状态 */
    tableLoading: Ref<boolean>
    /** 表单提交加载状态 */
    submitLoading: Ref<boolean>
    /** 编辑表单 页脚显示状态 */
    DialogVisibleFooterButton: Ref<boolean>
    /** 拆分元素 */
    ElementArrs: Ref<ElementArrsType>
    /** 要展示的表格列 */
    tableColumn: any
    /**存储原始数据 */
    originalData: Ref<[]>
    /**请求的数据 */
    tableData: Ref<[]>
    /**选中的数据 */
    multipleSelection: Ref<[]>
    /**上传的文件暂存 */
    uploadFiles: Ref<[]>
    /**初始化的值 */
    initSearchModel: { [x: string]: any }
    initEditModel: { [x: string]: any }
    /**路由传参的值 */
    searchFormValue: { [x: string]: any }
    /**搜索表单数据 */
    searchModel: { [x: string]: any }
    /** 原始编辑值 */
    editFormValue: { [x: string]: any }
    /**编辑表单数据 */
    editModel: { [x: string]: any }
}

/** 系统配置参数类型（在main中可以初始化的值） */
interface SystemParametersType {
    /** DeBug 开关 */
    debug: boolean

    /** 可操作最大高度 */
    mainMaxHeight: number
    /** 开启索引 */
    Table_isOpenIndex: boolean
    /** 开启多选 */
    Table_isOpenMultipleSelection: boolean
    /** 开启展开行 */
    Table_isOpenExpand: boolean
    /** 展开行数量 */
    Table_ExpandColumn: number
    /** 操作菜单宽度 */
    Table_MenuWidth: number
    /** 操作菜单对齐 */
    Table_MenuAlign: 'left' | 'center' | 'right'
    /** 分页大小 */
    Page_PageSizes: number[]
    /** 分页查询参数 */
    Page_PageParams: {
        /* 当前页数 */
        current: number
        /* 每页条数 */
        pageSize: number
        /* 总条数 */
        total: number
    }
    /** 分页布局 */
    Page_Layout: string
    /** 只有一页时隐藏分页 */
    Page_isHideOnSinglePage: boolean

    /** 开启搜索表单 */
    isOpenSearchForm: boolean
    /** 表单标签宽度 */
    Form_LabelWidth: number
    /** 对话框宽度 */
    Dialog_Width: number
    /** 开启全屏 */
    Dialog_isFullscreen: boolean
    /** 启ESC关闭对话框 */
    Dialog_isCloseOnPressEscape: boolean
    /** 开启点击modal关闭对话框 */
    Dialog_isCloseOnClickModal: boolean
    /** 开启页脚按钮 */
    Dialog_isFooterButton: boolean

    /** 开启在Mount后自动查询 */
    isAfterMountQuery: boolean

    /** 查询时api接口name名称 */
    ApiName: {
        query: string
        add: string
        edit: string
        delete: string
        deleteFlag: string
        remove: string
    }
    /** 渲染时页面节点权限参数 */
    Permission: {
        add: string
        edit: string
        delete: string
        deleteFlag: string
        remove: string
    }
    /** 所拥有的权限列表 */
    Permissions: string[]
}

/** 钩子名称类型 */
interface HooksNameType {
    /**提交之前 */
    beforeSubmitHandler: string
    /** 启用/禁用之前 */
    beforedisabledHandler: string
    /** 启用/禁用之后 */
    afterdisabledHandler: string
    /** 新增之前 */
    beforeAddHandler: string
    /** 编辑之前 */
    beforeEditHandler: string
    /** 查询之前 */
    beforeQueryHandler: string
    /** 查询之后 */
    afterQueryHandler: string
    /** 删除之前 */
    beforeDeteleHandler: string
    /** 删除之后 */
    afterDeteleHandler: string
    /** 批量删除之前 */
    beforeBatchDeteleHandler: string
    /** 批量删除之后 */
    afterBatchDeteleHandler: string
    // 表格懒加载之前
    beforeTableLazyHandler: string
}
/** 钩子方法类型 */
interface HooksConfigType {
    /** 钩子Debug */
    debug?: boolean
    /**提交之前 */
    beforeSubmitHandler?: ({ CrudState, Data }: { CrudState: CrusStateType; Data: any }) => boolean
    /** 启用/禁用之前 */
    beforedisabledHandler?: ({ CrudState, Data }: { CrudState: CrusStateType; Data: any }) => boolean
    /** 启用/禁用之后 */
    afterdisabledHandler?: ({ CrudState, Data }: { CrudState: CrusStateType; Data: any }) => boolean
    /** 新增之前 */
    beforeAddHandler?: ({ CrudState, Data }: { CrudState: CrusStateType; Data: any }) => boolean
    /** 编辑之前 */
    beforeEditHandler?: ({ CrudState, Data }: { CrudState: CrusStateType; Data: any }) => boolean
    /** 查询之前 */
    beforeQueryHandler?: ({ CrudState, Data }: { CrudState: CrusStateType; Data: any }) => boolean
    /** 查询之后 */
    afterQueryHandler?: ({ CrudState, Data }: { CrudState: CrusStateType; Data: any }) => boolean
    /** 删除之前 */
    beforeDeteleHandler?: ({ CrudState, Data }: { CrudState: CrusStateType; Data: any }) => boolean
    /** 删除之后 */
    afterDeteleHandler?: ({ CrudState, Data }: { CrudState: CrusStateType; Data: any }) => boolean
    /** 批量删除之前 */
    beforeBatchDeteleHandler?: ({ CrudState, Data }: { CrudState: CrusStateType; Data: any }) => boolean
    /** 批量删除之后 */
    afterBatchDeteleHandler?: ({ CrudState, Data }: { CrudState: CrusStateType; Data: any }) => boolean
    /** 表格懒加载之前 */
    beforeTableLazyHandler?: ({ CrudState, Data }: { CrudState: CrusStateType; Data: any }) => boolean
    [key: string]: any
}

/** Ref 暴露参数 */
declare type CrudStateType = {
    /** 分类原数据 */
    ElementArrs?: ElementArrsType
    /** 查询参数 */
    searchModel?: { [x: string]: any }
    searchFormValue?: { [x: string]: any }
    /** 编辑参数 */
    editModel?: { [x: string]: any }
    editFormValue?: { [x: string]: any }
    /** 上传文件暂存 */
    uploadFiles?: any[]
    /** 加载状态 */
    tableLoading?: boolean
    /** 分页状态 */
    Page_PageParams?: any

    Mode: 'add' | 'edit'

    /** 查询请求 */
    queryRecord?: () => void
    refresh?: () => void
    resetForm?: () => void
    addRecord?: () => void
    editRecord?: () => void
    handleSubmit?: () => void
    handleAdd?: () => void
    handleEdit?: () => void
    delFlag?: () => void
    deleteRecord?: () => void
    deleteMulRecord?: () => void
}
/** 数据监听参数 */
declare type watchDataType = {
    id: string
    data: {
        url: string
        name: string
        data: any
        key: string
        label: string | string[]
        value: string
    }[]
}
/** 对 columns 数据拆分*/
declare type ElementArrsType = {
    filter: columnsType
    editor: columnsType
    column: columnsType
    columns: columnsType
}

/** 数据源配置类型 */
declare type SourceOptType = {
    /** 搜索表单参数 并 排序 */
    filter: string[]
    /** 编辑表单参数 并 排序 */
    editor: string[]
    /** 表格列参数 并 排序 */
    column: string[]

    /** 具体参数 */
    columns: columnsType
}
/** 表单参数配置项 */
declare type columnsType = {
    [x: string]: columnsItemType
}
/** 表单参数配置项 */
declare type columnsItemType = {
    /** 渲染表单组件类型 */
    type: 'el-input' | 'el-input-number' | 'el-select' | 'el-date-picker' | 'el-time-picker' | 'el-upload' | 'el-cascader' | 'el-radio-group'
    /** 标签名 */
    label: string
    /** 在展开行中隐藏 */
    hidden?: boolean
    /** 校验规则 */
    rules?: any
    /** 在编辑表单中所占的列数  */
    span?: number
    /** 表格列对齐方式 */
    align?: string
    /** 搜索表单默认值 */
    searchDefaultValue?: any
    /** 新增表单默认值 */
    addDefaultValue?: any
    /** 新增/编辑模式 显示状态 */
    disabled?: (CrudState: CrusStateType, Mode: string) => boolean
    /** 新增/编辑模式 显示状态 */
    visible?: (CrudState: CrusStateType, Mode: string) => boolean
    /** 是否为范围选择器 */
    range?: boolean
    /** 文件上传时图片预览地址 */
    fileList?: { name: string; url: string }[]
    /** 表单组件配置参数 */
    formProps?: any
    /** 表格配置参数 */
    tableProps?: any
    /** 表格列组件化插槽配置 */
    columnSlot?: any
}
